////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.gowalla;

import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import org.json.JSONObject;


/***********************************************************************************************
Details about a single Gowalla friend
***********************************************************************************************/
public class GowallaFriend {

  /** The Gowalla user ID **/
  long userID = -1; // Custom Column "UserIdData"
  /** The users first name **/
  String firstName = null;
  /** The users last name **/
  String lastName = null;
  /** The URL to the photo of this user **/
  URL photo = null;
  /** The phone number of this user **/
  String phoneNumber = null;
  /** The email address of this user **/
  String emailAddress = null;
  /** The Twitter account of this user **/
  String twitter = null;
  /** The Facebook account of this user **/
  String facebook = null;
  /** The home city of this user **/
  String homeCity = null;
  /** The last location this user checked-in to **/
  String checkin = null;
  /** The last time this user checked-in to a venue **/
  long checkinTime = 0;
  /** The shoutout message that this user game at the last check-in **/
  String shoutout = null;
  /** The website of this user **/
  String website = null;
  /** The biography of this user **/
  String bio = null;
  /** The number of stamps this user has collected **/
  long stamps = -1;
  /** The number of pins this user has collected **/
  long pins = -1;
  /** The number of trips this user has completed **/
  long trips = -1;
  /** The number of items this user has collected **/
  long items = -1;


  /***********************************************************************************************
  Creates a new Gowalla friend from JSON data
  @param friend the JSON data for the friend
  ***********************************************************************************************/
  public GowallaFriend(JSONObject friend){
    toComponent(friend);
  }


  /***********************************************************************************************
  Creates a new Gowalla friend
  @param userID the Gowalla User ID
  @param firstName the first name of the user
  @param lastName the last name of the user
  @param photo the URL to the photo of this user
  @param homeCity the home city of the user
  ***********************************************************************************************/
  public GowallaFriend(long userID, String firstName, String lastName, URL photo, String homeCity){
    this.userID = userID;
    this.firstName = firstName;
    this.lastName = lastName;
    this.photo = photo;
    this.homeCity = homeCity;
  }


  /***********************************************************************************************
  Gets the biography of the user
  @return the <i>bio</i>
  ***********************************************************************************************/
  public String getBiography(){
    return bio;
  }


  /***********************************************************************************************
  Gets the last check-in location for this user
  @return the <i>checkin</i>
  ***********************************************************************************************/
  public String getCheckin(){
    return checkin;
  }


  /***********************************************************************************************
  Gets the last check-in time for this user
  @return the <i>checkinTime</i>
  ***********************************************************************************************/
  public long getCheckinTime(){
    return checkinTime;
  }


  /***********************************************************************************************
  Gets the email address of this user
  @return the <i>emailAddress</i>
  ***********************************************************************************************/
  public String getEmailAddress(){
    return emailAddress;
  }


  /***********************************************************************************************
  Gets the Facebook account of this user
  @return the <i>facebook</i> account
  ***********************************************************************************************/
  public String getFacebook(){
    return facebook;
  }


  /***********************************************************************************************
  Gets the first name of this user
  @return the <i>firstName</i>
  ***********************************************************************************************/
  public String getFirstName(){
    return firstName;
  }


  /***********************************************************************************************
  Gets the home city of this user
  @return the <i>homeCity</i>
  ***********************************************************************************************/
  public String getHomeCity(){
    return homeCity;
  }


  /***********************************************************************************************
  Gets the number of items the user has collected
  @return the <i>items</i>
  ***********************************************************************************************/
  public long getItems(){
    return items;
  }


  /***********************************************************************************************
  Gets the last name of this user
  @return the <i>lastName</i>
  ***********************************************************************************************/
  public String getLastName(){
    return lastName;
  }


  /***********************************************************************************************
  Gets the phone number of this user
  @return the <i>phoneNumber</i>
  ***********************************************************************************************/
  public String getPhoneNumber(){
    return phoneNumber;
  }


  /***********************************************************************************************
  Gets the URL for the photo of this user
  @return the <i>photo</i> URL
  ***********************************************************************************************/
  public URL getPhoto(){
    return photo;
  }


  /***********************************************************************************************
  Gets the number of pins the user has collected
  @return the <i>pins</i>
  ***********************************************************************************************/
  public long getPins(){
    return pins;
  }


  /***********************************************************************************************
  Gets the shoutout message for this user
  @return the <i>shoutout</i> message
  ***********************************************************************************************/
  public String getShoutout(){
    return shoutout;
  }


  /***********************************************************************************************
  Gets the number of stamps the user has collected
  @return the <i>stamps</i>
  ***********************************************************************************************/
  public long getStamps(){
    return stamps;
  }


  /***********************************************************************************************
  Gets the number of trips the user has completed
  @return the <i>trips</i>
  ***********************************************************************************************/
  public long getTrips(){
    return trips;
  }


  /***********************************************************************************************
  Gets the Twitter account of this user
  @return the <i>twitter</i> account
  ***********************************************************************************************/
  public String getTwitter(){
    return twitter;
  }


  /***********************************************************************************************
  Gets the user ID of this user
  @return the <i>userID</i>
  ***********************************************************************************************/
  public long getUserID(){
    return userID;
  }


  /***********************************************************************************************
  Gets the website of the user
  @return the <i>website</i>
  ***********************************************************************************************/
  public String getWebsite(){
    return website;
  }


  /***********************************************************************************************
  Sets the biography of the user
  @param bio the <i>bio</i>
  ***********************************************************************************************/
  public void setBiography(String bio){
    if (bio != null && bio.equals("null")) { // ignore 'null' values
      bio = null;
    }
    this.bio = bio;
  }


  /***********************************************************************************************
  Sets the last check-in location for this user
  @param checkin the last check-in location for this user
  ***********************************************************************************************/
  public void setCheckin(String checkin){
    if (checkin != null && checkin.equals("null")) { // ignore 'null' values
      checkin = null;
    }
    this.checkin = checkin;
  }


  /***********************************************************************************************
  Sets the last check-in time for this user
  @param checkinTime the last check-in time for this user
  ***********************************************************************************************/
  public void setCheckinTime(long checkinTime){
    this.checkinTime = checkinTime;
  }


  /***********************************************************************************************
  Sets the last check-in time for this user. Converts the String into a Long
  @param checkinTime the last check-in time for this user
  ***********************************************************************************************/
  public void setCheckinTime(String checkinTime){

    try {
      SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
      Date date = dateFormatter.parse(checkinTime);
      Calendar calendar = Calendar.getInstance();
      calendar.setTime(date);
      this.checkinTime = calendar.getTimeInMillis();
    }
    catch (Throwable t) {
      this.checkinTime = 0;
    }

  }


  /***********************************************************************************************
  Sets the email address of this user
  @param emailAddress the email address of this user
  ***********************************************************************************************/
  public void setEmailAddress(String emailAddress){
    if (emailAddress != null && emailAddress.equals("null")) { // ignore 'null' values
      emailAddress = null;
    }
    this.emailAddress = emailAddress;
  }


  /***********************************************************************************************
  Sets the Facebook account of this user
  @param facebook the Facebook account of this user
  ***********************************************************************************************/
  public void setFacebook(String facebook){
    if (facebook != null && facebook.equals("null")) { // ignore 'null' values
      facebook = null;
    }
    this.facebook = facebook;
  }


  /***********************************************************************************************
  Sets the first name of this user
  @param firstName the first name of this user
  ***********************************************************************************************/
  public void setFirstName(String firstName){
    if (firstName != null && firstName.equals("null")) { // ignore 'null' values
      firstName = null;
    }
    this.firstName = firstName;
  }


  /***********************************************************************************************
  Sets the home city of this user
  @param homeCity the home city of this user
  ***********************************************************************************************/
  public void setHomeCity(String homeCity){
    if (homeCity != null && homeCity.equals("null")) { // ignore 'null' values
      homeCity = null;
    }
    this.homeCity = homeCity;
  }


  /***********************************************************************************************
  Sets the number of items the user has collected
  @param items the <i>items</i>
  ***********************************************************************************************/
  public void setItems(long items){
    this.items = items;
  }


  /***********************************************************************************************
  Sets the last name of this user
  @param lastName the last name of this user
  ***********************************************************************************************/
  public void setLastName(String lastName){
    if (lastName != null && lastName.equals("null")) { // ignore 'null' values
      lastName = null;
    }
    this.lastName = lastName;
  }


  /***********************************************************************************************
  Sets the phone number of this user
  @param phoneNumber the phone number of this user
  ***********************************************************************************************/
  public void setPhoneNumber(String phoneNumber){
    if (phoneNumber != null && phoneNumber.equals("null")) { // ignore 'null' values
      phoneNumber = null;
    }
    this.phoneNumber = phoneNumber;
  }


  /***********************************************************************************************
  Sets the URL to the photo of this user
  @param photo the URL to the photo of this user
  ***********************************************************************************************/
  public void setPhoto(URL photo){
    if (photo != null && photo.equals("null")) { // ignore 'null' values
      photo = null;
    }
    this.photo = photo;
  }


  /***********************************************************************************************
  Sets the number of pins the user has collected
  @param pins the <i>pins</i>
  ***********************************************************************************************/
  public void setPins(long pins){
    this.pins = pins;
  }


  /***********************************************************************************************
  Sets the shoutout message for this user
  @param shoutout the last shoutout message for this user
  ***********************************************************************************************/
  public void setShoutout(String shoutout){
    if (shoutout != null && shoutout.equals("null")) { // ignore 'null' values
      shoutout = null;
    }
    this.shoutout = shoutout;
  }


  /***********************************************************************************************
  Sets the number of stamps the user has collected
  @param stamps the <i>stamps</i>
  ***********************************************************************************************/
  public void setStamps(long stamps){
    this.stamps = stamps;
  }


  /***********************************************************************************************
  Sets the number of trips the user has completed
  @param trips the <i>trips</i>
  ***********************************************************************************************/
  public void setTrips(long trips){
    this.trips = trips;
  }


  /***********************************************************************************************
  Sets the Twitter account of this user
  @param twitter the Twitter account of this user
  ***********************************************************************************************/
  public void setTwitter(String twitter){
    if (twitter != null && twitter.equals("null")) { // ignore 'null' values
      twitter = null;
    }
    this.twitter = twitter;
  }


  /***********************************************************************************************
  Sets the User ID of this user
  @param userID the User ID of this user
  ***********************************************************************************************/
  public void setUserID(long userID){
    this.userID = userID;
  }


  /***********************************************************************************************
  Sets the website of the user
  @param website the <i>website</i>
  ***********************************************************************************************/
  public void setWebsite(String website){
    if (website != null && website.equals("null")) { // ignore 'null' values
      website = null;
    }
    this.website = website;
  }


  /***********************************************************************************************
  Populates the FoursquareFriend from JSON data
  @param friend the JSON data for the friend
  ***********************************************************************************************/
  public void toComponent(JSONObject friend){
    try {

      // eg /users/2151593
      String url = friend.optString("url","");
      if (!url.equals("")) {
        try {
          this.userID = Long.parseLong(url.substring(7));
        }
        catch (Throwable t) {
          this.userID = -1;
        }
      }

      setFirstName(friend.optString("first_name",""));
      setLastName(friend.optString("last_name",""));
      setPhoto(new URL(friend.optString("image_url",null)));
      setHomeCity(friend.optString("hometown",""));
    }
    catch (Throwable t) {}

  }

}
