////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.gowalla.sync;

import java.util.Calendar;
import org.watto.android.R;
import org.watto.program.android.sync.gowalla.BatchOperation;
import org.watto.program.android.sync.gowalla.ContactOperations;
import org.watto.program.android.sync.gowalla.GowallaFriend;
import org.watto.program.android.sync.gowalla.GowallaQuery;
import org.watto.program.android.sync.gowalla.GlobalConstants;
import org.watto.program.android.sync.gowalla.datatype.BiographyData;
import org.watto.program.android.sync.gowalla.datatype.CheckInData;
import org.watto.program.android.sync.gowalla.datatype.ItemsData;
import org.watto.program.android.sync.gowalla.datatype.PinsData;
import org.watto.program.android.sync.gowalla.datatype.StampsData;
import org.watto.program.android.sync.gowalla.datatype.StatusData;
import org.watto.program.android.sync.gowalla.datatype.TripsData;
import org.watto.program.android.sync.gowalla.datatype.UserIdData;
import org.watto.program.android.sync.gowalla.datatype.WebsiteData;
import android.accounts.Account;
import android.accounts.AccountManager;
import android.content.AbstractThreadedSyncAdapter;
import android.content.ContentProviderClient;
import android.content.ContentResolver;
import android.content.ContentUris;
import android.content.Context;
import android.content.SyncResult;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.provider.ContactsContract;
import android.provider.ContactsContract.CommonDataKinds.Email;
import android.provider.ContactsContract.CommonDataKinds.GroupMembership;
import android.provider.ContactsContract.CommonDataKinds.Im;
import android.provider.ContactsContract.CommonDataKinds.Phone;
import android.provider.ContactsContract.CommonDataKinds.Photo;
import android.provider.ContactsContract.CommonDataKinds.StructuredName;
import android.provider.ContactsContract.CommonDataKinds.StructuredPostal;
import android.provider.ContactsContract.Data;
import android.provider.ContactsContract.Groups;
import android.provider.ContactsContract.RawContacts;
import android.provider.ContactsContract.RawContacts.Entity;


/***********************************************************************************************
A <code>SyncAdapter</code> implementation for syncing <code>SyncAdapter</code> contacts to the
platform <code>ContactOperations</code> provider.
***********************************************************************************************/
public class SyncAdapter extends AbstractThreadedSyncAdapter {

  /** Access to the sync accounts on this device **/
  AccountManager accountManager;
  Context context;


  /***********************************************************************************************
  Constructor
  ***********************************************************************************************/
  public SyncAdapter(Context context, boolean autoInitialize){
    super(context,autoInitialize);
    this.context = context;
    this.accountManager = AccountManager.get(context);
  }


  /***********************************************************************************************
  Performs a synchronisation for this account. SyncAdapter-specific parameters may be specified in
  extras, which is guaranteed to not be null.
  @param account the account that should be synced 
  @param extras SyncAdapter-specific parameters 
  @param authority the authority of this sync request 
  @param provider a ContentProviderClient that points to the ContentProvider for this authority 
  @param syncResult SyncAdapter-specific parameters  
  ***********************************************************************************************/
  public void onPerformSync(Account account,Bundle extras,String authority,ContentProviderClient provider,SyncResult syncResult){
    try {
      // Get the list of friends, and sync them with the Android Contacts
      GowallaFriend[] friends = GowallaQuery.getFriends();
      syncContacts(context,account.name,friends);
    }
    catch (Throwable t) {
      syncResult.stats.numIoExceptions++;
    }
  }


  /***********************************************************************************************
  Adds a single contact to the platform contacts provider
  @param context the Authenticator Activity context
  @param accountName the account the contact belongs to
  @param friend the user to add
  @param rawGroupId the unique Id for the "FourSquare" group
  @param rawStatusId the unique Id for the "FourSquare" status message
  @param batchOperations any batch operations to perform when creating the new contact 
  ***********************************************************************************************/
  public static void addContact(Context context,String accountName,GowallaFriend friend,long rawGroupId,long rawStatusId,BatchOperation batchOperations){
    // Creates a new ContactOperation batch process
    int userID = -1; // the raw userID will be populated when the contact is created
    ContactOperations contactOperation = ContactOperations.createNewContact(context,userID,accountName,batchOperations);

    // Adds the content to the operation
    contactOperation.addUserId(friend.getUserID());
    contactOperation.addName(friend.getFirstName(),friend.getLastName());
    contactOperation.addPhoto(friend.getPhoto());
    contactOperation.addPhone(friend.getPhoneNumber());
    contactOperation.addEmail(friend.getEmailAddress());
    contactOperation.addTwitter(friend.getTwitter());
    contactOperation.addFacebook(friend.getFacebook());
    contactOperation.addHomeCity(friend.getHomeCity());
    contactOperation.addStatus(friend.getShoutout(),friend.getCheckin());
    contactOperation.addCheckin(friend.getCheckin(),friend.getCheckinTime());
    contactOperation.addWebsite(friend.getWebsite());
    contactOperation.addBiography(friend.getBiography());
    contactOperation.addStamps(friend.getStamps());
    contactOperation.addPins(friend.getPins());
    contactOperation.addTrips(friend.getTrips());
    contactOperation.addItems(friend.getItems());

    // Note: Don't add the status icon here, as the data for this user doesn't exist yet.
    // It is done in the syncContacts() instead.
    //contactOperation.addStatusIcon(friend.getShoutout(),friend.getCheckin(),rawStatusId);

    contactOperation.addGroupMembership(rawGroupId);
  }


  /***********************************************************************************************
  Synchronise raw contacts
  @param context The context of Authenticator Activity
  @param accountName The username for the account
  @param users The list of users
  ***********************************************************************************************/
  public static void addGroup(Context context,String accountName,BatchOperation batchOperation){

    // Creates a new ContactOperation batch process
    int groupID = -1; // the raw groupID will be populated when the group is created
    ContactOperations contactOperation = ContactOperations.createNewGroup(context,groupID,accountName,batchOperation);

    String title = context.getString(R.string.contact_group_title);
    String notes = context.getString(R.string.contact_group_notes);
    contactOperation.addGroup(title,notes);
  }


  /***********************************************************************************************
  Deletes a contact from the platform contacts provider
  @param context the Authenticator Activity context
  @param rawContactId the unique Id for this rawContact in contacts provider
  @param batchOperations any batch operations to perform when deleting the contact 
  ***********************************************************************************************/
  public static void deleteContact(Context context,long rawContactId,BatchOperation batchOperation){
    batchOperation.add(ContactOperations.addDeleteOperation(ContentUris.withAppendedId(RawContacts.CONTENT_URI,rawContactId),true).build());
  }


  /***********************************************************************************************
  Gets the platforms' rawContactID for the Foursquare group
  @param resolver the ContentResolver for querying the platform contacts
  @return the rawGroupID for the group, or <b>0</b> if no matching group was found
  ***********************************************************************************************/
  public static long getGroupID(ContentResolver resolver){

    String[] columns = new String[]{Groups._ID};
    String whereClause = ContactsContract.Groups.ACCOUNT_TYPE + "='" + GlobalConstants.ACCOUNT_TYPE + "'" + " AND " + ContactsContract.Groups.ACCOUNT_NAME + "='" + GlobalConstants.ACCOUNT_NAME + "'";

    Cursor c = resolver.query(ContactsContract.Groups.CONTENT_URI,columns,whereClause,null,null);

    long rawGroupID = -1;
    try {
      if (c != null && c.moveToFirst()) {
        rawGroupID = c.getLong(0);
      }
    }
    finally {
      if (c != null) {
        c.close();
      }
    }
    return rawGroupID;
  }


  /***********************************************************************************************
  Gets the platforms' rawContactID for the contact <code>FoursquareFriend.getUserID()</code>
  @param resolver the ContentResolver for querying the platform contacts
  @param userID the <code>FoursquareFriend.getUserID()</code> to lookup
  @return the rawContactID for the contact, or <b>0</b> if no matching contact was found
  ***********************************************************************************************/
  public static long getRawContactID(ContentResolver resolver,long userID){

    //String[] columns = new String[]{RawContacts._ID};
    String[] columns = new String[]{Data.RAW_CONTACT_ID};
    String whereClause = RawContacts.ACCOUNT_TYPE + "='" + GlobalConstants.ACCOUNT_TYPE + "' AND " + Data.MIMETYPE + "='" + UserIdData.CONTENT_ITEM_TYPE + "' AND " + UserIdData.DATA + "=" + userID;

    long rawContactID = 0;
    Cursor c = resolver.query(Data.CONTENT_URI,columns,whereClause,null,null);
    try {
      if (c != null && c.moveToFirst()) {
        rawContactID = c.getLong(0);
      }
    }
    finally {
      if (c != null) {
        c.close();
      }
    }
    return rawContactID;
  }


  /***********************************************************************************************
  Gets the platforms' rawContactID for the Foursquare status
  @param resolver the ContentResolver for querying the platform contacts
  @return the rawContactID for the contact, or <b>0</b> if no matching contact was found
  ***********************************************************************************************/
  public static long getStatusDataID(ContentResolver resolver,long rawContactId){
    if (rawContactId == -1) {
      return -1;
    }

    Uri rawContactUri = ContentUris.withAppendedId(RawContacts.CONTENT_URI,rawContactId);
    Uri entityUri = Uri.withAppendedPath(rawContactUri,Entity.CONTENT_DIRECTORY);

    String[] columns = new String[]{Entity.DATA_ID,Entity.MIMETYPE};
    String whereClause = Entity.MIMETYPE + "='" + StatusData.CONTENT_ITEM_TYPE + "'";

    long rawStatusId = -1;
    try {
      //Cursor c = resolver.query(entityUri,columns,null,null,null);
      Cursor c = resolver.query(entityUri,columns,whereClause,null,null);
      try {
        while (c != null && c.moveToNext()) {
          String mimetype = c.getString(1);
          // Store the Status against the Status data of the user
          if (mimetype.equals(StatusData.CONTENT_ITEM_TYPE)) {
            rawStatusId = c.getLong(0);
            return rawStatusId;
          }

        }
      }
      finally {
        if (c != null) {
          c.close();
        }
      }
    }
    catch (Throwable t) {}
    return rawStatusId;
  }


  /***********************************************************************************************
  Synchronise raw contacts
  @param context The context of Authenticator Activity
  @param accountName The name of the account
  @param users The list of users
  ***********************************************************************************************/
  public static synchronized void syncContacts(Context context,String accountName,GowallaFriend[] friends){
    if (friends == null || friends.length <= 0) {
      return;
    }

    long userID;
    long rawContactId = 0;
    long rawGroupId = -1;
    long rawStatusId = -1;

    ContentResolver resolver = context.getContentResolver();
    BatchOperation batchOperation = new BatchOperation(context,resolver);

    rawGroupId = getGroupID(resolver);
    if (rawGroupId == -1) {
      // Create the group
      addGroup(context,accountName,batchOperation);
      // Get the group ID
      rawGroupId = getGroupID(resolver);
    }

    int friendCount = friends.length;
    for (int i = 0;i < friendCount;i++) {
      GowallaFriend friend = friends[i];

      userID = friend.getUserID();
      // Check to see if the contact needs to be inserted or updated
      rawContactId = getRawContactID(resolver,userID);

      // Update the users Status icon and message
      rawStatusId = getStatusDataID(resolver,rawContactId);

      if (rawContactId != 0) {
        // update contact
        updateContact(context,resolver,accountName,friend,rawContactId,rawGroupId,rawStatusId,batchOperation);
      }
      else {
        // add new contact
        addContact(context,accountName,friend,rawGroupId,rawStatusId,batchOperation);
        // create the contact now, so we can update the status
        batchOperation.execute();
        // now get the rawContactID and statusDataID, then update the status
        rawContactId = getRawContactID(resolver,userID);
        rawStatusId = getStatusDataID(resolver,rawContactId);
        new ContactOperations(context,batchOperation).addStatusIcon(friend.getShoutout(),friend.getCheckin(),rawStatusId);
      }

      // Execute and clear the batch operations if it gets too large
      if (batchOperation.size() >= 50) {
        batchOperation.execute();
      }
    }

    // Execute the batch operations
    batchOperation.execute();
  }


  /***********************************************************************************************
  Updates a single contact to the platform contacts provider.
  @param context the Authenticator Activity context
  @param resolver the ContentResolver to use
  @param accountName the account the contact belongs to
  @param friend the user being updated
  @param rawContactId the unique Id for this rawContact in contacts provider
  @param rawGroupId the unique Id for the "FourSquare" group
  @param rawStatusId the unique Id for the "FourSquare" status message
  ***********************************************************************************************/
  public static void updateContact(Context context,ContentResolver resolver,String accountName,GowallaFriend friend,long rawContactId,long rawGroupId,long rawStatusId,BatchOperation batchOperation){
    Uri uri;

    long oldUserID = -1;
    String oldFirstName = null;
    String oldLastName = null;
    //URL oldPhoto = null; // SPECIAL CASE - See lower down!
    boolean photoUpdated = false;
    String oldPhone = null;
    String oldEmail = null;
    String oldTwitter = null;
    String oldFacebook = null;
    String oldHomeCity = null;
    String oldCheckin = null;
    String oldShoutout = null;
    long oldCheckinTime = 0;

    String oldWebsite = null;
    String oldBiography = null;
    long oldStamps = -1;
    long oldPins = -1;
    long oldTrips = -1;
    long oldItems = -1;

    long oldGroupMembership = -1;

    ContactOperations contactOperation = ContactOperations.updateExistingContact(context,rawContactId,batchOperation);

    // BUG FIX!
    // There was a bug in 1.0 and 1.1 where a new photo would be added to the contact every day!
    // This code here is to detect and remove all the photo details for the contact, if there are
    // more than 1, so that it can update the existing photo rather than adding new ones!
    String[] photoBugColumns = new String[]{Data._ID};
    String photoBugWhereClause = Data.RAW_CONTACT_ID + "=" + rawContactId + " AND " + Data.MIMETYPE + "='" + Photo.CONTENT_ITEM_TYPE + "'";
    Cursor photoBugC = resolver.query(Data.CONTENT_URI,photoBugColumns,photoBugWhereClause,null,null);
    int count = photoBugC.getCount();
    if (count > 1) {
      // remove all the photo entries
      while (photoBugC.moveToNext()) {
        long columnId = photoBugC.getLong(0);
        uri = ContentUris.withAppendedId(Data.CONTENT_URI,columnId);
        contactOperation.deletePhoto(uri);
      }
      // Clear the photos right now, so we can get the correct Photos in the next Cursor SELECT
      batchOperation.execute();
    }
    photoBugC.close();

    // Now the real update query...
    String[] columns = new String[]{Data._ID,Data.MIMETYPE,Data.DATA1,Data.DATA2,Data.DATA3,Data.DATA4,Data.DATA5,Data.DATA6,Data.DATA7,Data.DATA8,Data.DATA9,Data.DATA10,Data.SYNC1};
    String whereClause = Data.RAW_CONTACT_ID + "=" + rawContactId;
    Cursor c = resolver.query(Data.CONTENT_URI,columns,whereClause,null,null);

    try {
      while (c.moveToNext()) {
        long columnId = c.getLong(0);
        uri = ContentUris.withAppendedId(Data.CONTENT_URI,columnId);

        String mimeType = c.getString(1);

        if (mimeType.equals(UserIdData.CONTENT_ITEM_TYPE)) {
          oldUserID = c.getLong(2);
          contactOperation.updateUserId(friend.getUserID(),oldUserID,uri);
        }
        else if (mimeType.equals(WebsiteData.CONTENT_ITEM_TYPE)) {
          oldWebsite = c.getString(2);
          contactOperation.updateWebsite(friend.getWebsite(),oldWebsite,uri);
        }
        else if (mimeType.equals(BiographyData.CONTENT_ITEM_TYPE)) {
          oldBiography = c.getString(2);
          contactOperation.updateBiography(friend.getBiography(),oldBiography,uri);
        }
        else if (mimeType.equals(StampsData.CONTENT_ITEM_TYPE)) {
          oldStamps = c.getLong(2);
          contactOperation.updateStamps(friend.getStamps(),oldStamps,uri);
        }
        else if (mimeType.equals(PinsData.CONTENT_ITEM_TYPE)) {
          oldPins = c.getLong(2);
          contactOperation.updatePins(friend.getPins(),oldPins,uri);
        }
        else if (mimeType.equals(TripsData.CONTENT_ITEM_TYPE)) {
          oldTrips = c.getLong(2);
          contactOperation.updateTrips(friend.getTrips(),oldTrips,uri);
        }
        else if (mimeType.equals(ItemsData.CONTENT_ITEM_TYPE)) {
          oldItems = c.getLong(2);
          contactOperation.updateItems(friend.getItems(),oldItems,uri);
        }
        else if (mimeType.equals(StructuredName.CONTENT_ITEM_TYPE)) {
          oldFirstName = c.getString(3);
          oldLastName = c.getString(4);
          contactOperation.updateName(friend.getFirstName(),oldFirstName,friend.getLastName(),oldLastName,uri);
        }
        else if (mimeType.equals(Phone.CONTENT_ITEM_TYPE)) {
          int type = c.getInt(3);
          String label = c.getString(4);

          oldPhone = c.getString(2);

          if (type == Phone.TYPE_MOBILE) {
            // v1.0 and 1.1 use TYPE_MOBILE. v1.2+ use TYPE_CUSTOM with a 'Foursquare Phone' label.
            // This will detect the phone being stored as TYPE_MOBILE and change it to TYPE_CUSTOM.
            contactOperation.convertPhoneToCustom(friend.getPhoneNumber(),uri);
          }
          else if (type == Phone.TYPE_CUSTOM && label.equals(context.getString(R.string.label_phone))) {
            contactOperation.updatePhone(friend.getPhoneNumber(),oldPhone,uri);
          }
        }
        else if (mimeType.equals(StructuredPostal.CONTENT_ITEM_TYPE)) {
          int type = c.getInt(3);
          String label = c.getString(4);

          oldHomeCity = c.getString(2);

          if (type == StructuredPostal.TYPE_HOME) {
            // v1.0 and 1.1 use TYPE_HOME. v1.2+ use TYPE_CUSTOM with a 'Foursquare Home City' label.
            // This will detect the address being stored as TYPE_HOME and change it to TYPE_CUSTOM.
            contactOperation.convertHomeCityToCustom(friend.getHomeCity(),uri);
          }
          else if (type == StructuredPostal.TYPE_CUSTOM && label.equals(context.getString(R.string.label_homecity))) {
            oldHomeCity = c.getString(2);
            contactOperation.updateHomeCity(friend.getHomeCity(),oldHomeCity,uri);
          }
        }
        else if (mimeType.equals(StatusData.CONTENT_ITEM_TYPE)) {
          oldShoutout = c.getString(2);
          contactOperation.updateStatus(friend.getShoutout(),oldShoutout,friend.getCheckin(),uri);
          // Update the status icon too
          contactOperation.updateStatusIcon(friend.getShoutout(),oldShoutout,friend.getCheckin(),rawStatusId);
        }
        else if (mimeType.equals(CheckInData.CONTENT_ITEM_TYPE)) {
          oldCheckin = c.getString(3);
          try {
            oldCheckinTime = Long.parseLong(c.getString(12));
          }
          catch (NumberFormatException e) {
            // When upgrading from 1.0 to 1.1, users don't already have the oldCheckinTime
          }
          contactOperation.updateCheckin(friend.getCheckin(),oldCheckin,friend.getCheckinTime(),oldCheckinTime,uri);
        }
        else if (mimeType.equals(GroupMembership.CONTENT_ITEM_TYPE)) {
          // The group never changes, so no update operation is required here. 
          // Just set the oldGroupMembership to a value so its not added to the group down below.
          oldGroupMembership = c.getLong(2);
        }
        else if (mimeType.equals(Email.CONTENT_ITEM_TYPE)) {
          oldEmail = c.getString(2);
          contactOperation.updateEmail(friend.getEmailAddress(),oldEmail,uri);
        }
        else if (mimeType.equals(Photo.CONTENT_ITEM_TYPE)) {
          boolean timeExpired = false;
          try {
            long oldTimestamp = Long.parseLong(c.getString(12));
            long timestamp = Calendar.getInstance().getTimeInMillis();

            // Check that the photo is at least 1 day old

            long timeDifference = timestamp - oldTimestamp;
            if (timeDifference > 82800000) { // 23 hours in milliseconds
              timeExpired = true;
            }

          }
          catch (NumberFormatException e) {
            timeExpired = false;
          }

          if (timeExpired) {
            contactOperation.updatePhoto(friend.getPhoto(),uri);
          }

          // If it gets here, this contact already has a photo, so don't want to add a new photo!
          photoUpdated = true;
        }
        else if (mimeType.equals(Im.CONTENT_ITEM_TYPE)) {
          String customProvider = c.getString(7);

          if (customProvider.equals("Twitter")) {
            oldTwitter = c.getString(2);
            contactOperation.updateTwitter(friend.getTwitter(),oldTwitter,uri);
          }
          else if (customProvider.equals("Facebook")) {
            oldFacebook = c.getString(2);
            contactOperation.updateFacebook(friend.getFacebook(),oldFacebook,uri);
          }
        }
      }
    }
    finally {
      c.close();
    }

    // For all unchanged values, add them from the <i>friend</i>
    if (oldUserID == -1) {
      contactOperation.addUserId(friend.getUserID());
    }
    if (oldWebsite == null) {
      contactOperation.addWebsite(friend.getWebsite());
    }
    if (oldBiography == null) {
      contactOperation.addBiography(friend.getBiography());
    }
    if (oldStamps == -1) {
      contactOperation.addStamps(friend.getStamps());
    }
    if (oldPins == -1) {
      contactOperation.addPins(friend.getPins());
    }
    if (oldTrips == -1) {
      contactOperation.addTrips(friend.getTrips());
    }
    if (oldItems == -1) {
      contactOperation.addItems(friend.getItems());
    }
    if (oldFirstName == null && oldLastName == null) {
      contactOperation.addName(friend.getFirstName(),friend.getLastName());
    }
    if (oldPhone == null) {
      contactOperation.addPhone(friend.getPhoneNumber());
    }
    if (oldEmail == null) {
      contactOperation.addEmail(friend.getEmailAddress());
    }
    if (oldTwitter == null) {
      contactOperation.addTwitter(friend.getTwitter());
    }
    if (oldFacebook == null) {
      contactOperation.addFacebook(friend.getFacebook());
    }
    if (oldHomeCity == null) {
      contactOperation.addHomeCity(friend.getHomeCity());
    }
    if (oldCheckin == null) {
      contactOperation.addCheckin(friend.getCheckin(),friend.getCheckinTime());
    }
    if (oldShoutout == null) {
      contactOperation.addStatus(friend.getShoutout(),friend.getCheckin());
      // Update the status icon too
      contactOperation.addStatusIcon(friend.getShoutout(),friend.getCheckin(),rawStatusId);
    }

    if (oldGroupMembership == -1) {
      contactOperation.addGroupMembership(rawGroupId);
    }

    if (!photoUpdated) {
      contactOperation.addPhoto(friend.getPhoto());
    }

  }
}
